package net.wanji.utc.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.common.baseentity.BaseCrossInfo;
import net.wanji.utc.common.commonentity.HttpRequest;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.service.HkGetSignalMethodService;
import net.wanji.utc.vo.CrossLanesVo;
import net.wanji.utc.vo.signal.SignalLightStateVo;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static net.wanji.utc.common.constant.Constants.HK_CODE_KEY;
import static net.wanji.utc.common.constant.Constants.HK_SUCCESS_CODE;

/**
 * @author hfx
 * Created on 2020/09/10
 * @description: [海康信号灯厂商接口service实现]
 */
@Slf4j
@Service
public class HkGetSignalMethodServiceImpl implements HkGetSignalMethodService {
    @Value("${signal.manufacturer.hk.artemisPath}")
    private String artemisPath;

    @Resource
    private ArtemisConfig artemisConfig;

    /**
     * 获取路口信号机的实时灯态数据
     */
    @Override
    @SuppressWarnings("unchecked")
    public List<SignalLightStateVo> queryHkSignalInfo(List<BaseCrossInfo> baseSignals) {
        List<SignalLightStateVo> result = new ArrayList<>();
        SignalLightStateVo infoVo;
        try {
            Map<String, String> path = getPathMapByApiCode("querySignalInfo");
            for (BaseCrossInfo entity : baseSignals) {
                JSONObject jsonBody = new JSONObject();
                jsonBody.put("pageNo", 1);
                jsonBody.put("pageSize", 20);

                JSONObject searchObj = new JSONObject();
                JSONArray crossCodes = new JSONArray();
                crossCodes.add(entity.getCode());
                searchObj.put("crossCodes", crossCodes);
                jsonBody.put("searchObj", searchObj);
                String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, jsonBody.toJSONString(), null, null, "application/json", null);
                JSONObject object = JSON.parseObject(strResult);
                //根据路口ID 找到对应通道号上的灯组方向
                List<CrossLanesVo> crossDetailInfo = Constants.getTelesemeLaneRealMap().get(entity.getCode());
                if (HK_SUCCESS_CODE.equals(object.getInteger(HK_CODE_KEY))) {
                    JSONObject data = object.getJSONObject("data");
                    JSONArray list = data.getJSONArray("list");
                    for (int i = 0; i < list.size(); i++) {
                        infoVo = new SignalLightStateVo();
                        JSONObject jsonObject = list.getJSONObject(i);

                        String patternNo = jsonObject.getString("patternNo");               // 方案编号
                        JSONArray rings = jsonObject.getJSONArray("rings");                 // 环列表
                        JSONArray channelState = jsonObject.getJSONArray("channelState");   // 灯组列表
                        infoVo.setRunMode(jsonObject.getString("controlType"));             // 运行模式
                        infoVo.setPhasePlanId(patternNo);                                       // 相位方案号
                        int cycleLen = jsonObject.getIntValue("cycle");                     // 周期时长
                        int curRunTime = jsonObject.getIntValue("curRunTime");              // 当前运行时长
                        int cycleCountDown = cycleLen - curRunTime;                             // 周期剩余时长
                        infoVo.setCode(entity.getCode());
                        infoVo.setManufacturerCode(entity.getManufacturerCode());
                        infoVo.setCycleCountDown(cycleCountDown);
                        infoVo.setCycleLen(cycleLen);

                        Map<String, Object> dirLampGroupMap = new HashMap<>();
                        Map<String, Object> phaseMap = new HashMap<>();
                        // 循环构建灯组对象（从路口详情接口中获取灯组关系）
                        for (int k = 0; k < channelState.size(); k++) {
                            JSONObject cobj = channelState.getJSONObject(k);
                            int channelNo = cobj.getIntValue("channelNo");
                            String color = transferHKLampGroup(cobj.getIntValue("state"));
                            for (CrossLanesVo vo : crossDetailInfo) {
                                if (vo.getChannelNo() == channelNo) {
                                    String direction = handleDirection(vo.getDirection().toString());
                                    String trun = handleTurnTypeByDt(vo.getTurn());
                                    if (dirLampGroupMap.get(direction) != null) {
                                        Map<String, String> mstr = (Map<String, String>) dirLampGroupMap.get(direction);
                                        mstr.put(trun, color);
                                        dirLampGroupMap.put(direction, mstr);
                                    } else {
                                        Map<String, String> lamp = new HashMap<>();
                                        lamp.put(trun, color);
                                        dirLampGroupMap.put(direction, lamp);
                                    }
                                }
                            }
                        }

                        //循环相位列表
                        for (int j = 0; j < rings.size(); j++) {

                            JSONObject jsonObj = rings.getJSONObject(j);
                            JSONArray phaseList = jsonObj.getJSONArray("phaseList");
                            for (int k = 0; k < phaseList.size(); k++) {
                                JSONObject phase = phaseList.getJSONObject(k);
                                String phaseNo = phase.getString("phaseNo");
                                Integer vehicleStatus = phase.getInteger("vehicleStatus");
                                Integer phaseLength = phase.getInteger("phaseLength");
                                Integer red = phase.getInteger("red");
                                Integer yellow = phase.getInteger("yellow");
                                Integer green = phaseLength - yellow - red;
                                phaseMap.put(phaseNo, green);
                                if (2 != vehicleStatus) {
                                    infoVo.setPhaseId(phaseNo);
                                }
                            }
                        }
                        infoVo.setDirLampGroupMap(dirLampGroupMap);
                        infoVo.setPhaseMap(phaseMap);
                        result.add(infoVo);
                    }
                }
            }
        } catch (Exception e) {
            log.error("系统繁忙，服务器端内部错误！", e);
        }
        return result;
    }

    private Map<String, String> getPathMapByApiCode(String apiCode) {
        Map<String, String> res = new HashMap<>();
        HttpRequest httpRequest = new HttpRequest(BasicEnum.ManufacturerEnum.HK.getCode(), apiCode);
        res.put("http://", artemisPath + httpRequest.getUrl());
        return res;
    }

    /**
     * 灯组颜色转换
     */
    private String transferHKLampGroup(int state) {
        String color;
        switch (state) {
            case 1:
                color = "green";
                break;
            case 2:
            case 6:
                color = "red";
                break;
            case 3:
                color = "yellow";
                break;
            case 4:
                color = "greenFlash";
                break;
            case 5:
                color = "yellowFlash";
                break;
            case 7:
                color = "lightsOut";
                break;
            case 8:
                color = "redFlash";
                break;
            default:
                color = "";
                break;
        }
        return color;
    }

    /**
     * 处理方向
     * 车道进口方向，方向：北 1 ,东北 2 ,东 3 ,东南 4 ,南 5 ,西南 6 ,西 7 ,西北 8
     *
     * @param direction 海康方向: 0 其他,1 由南向北,2 由北向南,3 由东向西,4 由西向东,5 西南向东北,6 西北向东南,7 东南向西北,8 东北向西南,9 北行人,10 南行人,11 西行人,12 东行人
     */
    private String handleDirection(String direction) {
        String nDirection;
        switch (direction) {
            case "1":
            case "10":
                nDirection = "5";
                break;
            case "2":
            case "9":
                nDirection = "1";
                break;
            case "3":
            case "12":
                nDirection = "3";
                break;
            case "4":
            case "11":
                nDirection = "7";
                break;
            case "5":
                nDirection = "6";
                break;
            case "6":
                nDirection = "8";
                break;
            case "7":
                nDirection = "4";
                break;
            case "8":
                nDirection = "2";
                break;
            //其他
            default:
                nDirection = "0";
        }
        return nDirection;
    }

    /**
     * 灯态车道转向
     * 处理转向 1.左转、2.直行、3.右转、4.掉头、5.向左合流、6.向右合流、7.右掉头、8.左转直行、9.直行右转、
     * 10.左转掉头、11.右转掉头、12.直行掉头、13.直行右掉头、14.左转右转、15.左转直行右转、16.左转右转掉头、17.左转右转右掉头、18.左转直行掉
     *
     * @param movement 海康转向: 1 直行,2 左转,3 右转,4 掉头,5 直左,6 直右,7 行人,8 左右,9 通行（左直右）,10 左转掉头
     */
    private String handleTurnTypeByDt(int movement) {
        //处理海康转向 5,6,7,8,9,10 暂时不处理
        String turnType;
        switch (movement) {
            case 1:
                turnType = "2";
                break;
            case 2:
                turnType = "1";
                break;
            case 3:
                turnType = "3";
                break;
            case 4:
                turnType = "4";
                break;
            case 5:
                turnType = "8";
                break;
            case 6:
                turnType = "9";
                break;
            case 8:
                turnType = "14";
                break;
            case 9:
                turnType = "15";
                break;
            case 10:
                turnType = "10";
                break;
            default:
                turnType = "0";
        }
        return turnType;
    }
}
